@@ -53,8 +53,9 @@ extern "C" {
 #endif /* __cplusplus */
 
 /* SphinxBase headers. */
-#include <mmio.h>
-#include <cmd_ln.h>
+#include <sphinxbase/mmio.h>
+#include <sphinxbase/cmd_ln.h>
+#include <pocketsphinx_export.h>
 
 #include "mdef.h"
 
@@ -79,13 +80,13 @@ struct mdef_entry_s {
 	union {
 		/**< CI phone information - attributes (just "filler" for now) */
 		struct {
-			int8 filler;
-			int8 reserved[3];
+			uint8 filler;
+			uint8 reserved[3];
 		} ci;
 		/**< CD phone information - context info. */
 		struct {
-			int8 wpos;
-			int8 ctx[3]; /**< quintphones will require hacking */
+			uint8 wpos;
+			uint8 ctx[3]; /**< quintphones will require hacking */
 		} cd;
 	} info;
 } ATTRIBUTE_PACKED;
@@ -167,18 +168,22 @@ struct bin_mdef_s {
 /**
  * Read a binary mdef from a file.
  */
+POCKETSPHINX_EXPORT
 bin_mdef_t *bin_mdef_read(cmd_ln_t *config, const char *filename);
 /**
  * Read a text mdef from a file (creating an in-memory binary mdef).
  */
+POCKETSPHINX_EXPORT
 bin_mdef_t *bin_mdef_read_text(cmd_ln_t *config, const char *filename);
 /**
  * Write a binary mdef to a file.
  */
+POCKETSPHINX_EXPORT
 int bin_mdef_write(bin_mdef_t *m, const char *filename);
 /**
  * Write a binary mdef to a text file.
  */
+POCKETSPHINX_EXPORT
 int bin_mdef_write_text(bin_mdef_t *m, const char *filename);
 /**
  * Retain a pointer to a bin_mdef_t.
