@@ -52,9 +52,9 @@
 #include <assert.h>
 
 /* SphinxBase headers. */
-#include <prim_type.h>
-#include <err.h>
-#include <ckd_alloc.h>
+#include <sphinxbase/prim_type.h>
+#include <sphinxbase/err.h>
+#include <sphinxbase/ckd_alloc.h>
 
 /* Local headers. */
 #include "fsg_search_internal.h"
@@ -73,9 +73,8 @@ fsg_history_init(fsg_model_t * fsg, dict_t *dict)
     h->fsg = fsg;
     h->entries = blkarray_list_init();
 
-    if (fsg) {
-        if (dict)
-            h->n_ciphone = bin_mdef_n_ciphone(dict->mdef);
+    if (fsg && dict) {
+        h->n_ciphone = bin_mdef_n_ciphone(dict->mdef);
         h->frame_entries =
             (glist_t **) ckd_calloc_2d(fsg_model_n_state(fsg),
                                        bin_mdef_n_ciphone(dict->mdef),
@@ -126,9 +125,8 @@ fsg_history_set_fsg(fsg_history_t *h, fsg_model_t *fsg, dict_t *dict)
     h->frame_entries = NULL;
     h->fsg = fsg;
 
-    if (fsg) {
-        if (dict)
-            h->n_ciphone = bin_mdef_n_ciphone(dict->mdef);
+    if (fsg && dict) {
+        h->n_ciphone = bin_mdef_n_ciphone(dict->mdef);
         h->frame_entries =
             (glist_t **) ckd_calloc_2d(fsg_model_n_state(fsg),
                                        bin_mdef_n_ciphone(dict->mdef),
@@ -298,3 +296,33 @@ void
 fsg_history_utt_end(fsg_history_t * h)
 {
 }
+
+void
+fsg_history_print(fsg_history_t *h, dict_t *dict) 
+{
+    int bpidx, bp;
+    
+    for (bpidx = 0; bpidx < blkarray_list_n_valid(h->entries); bpidx++) {
+        bp = bpidx;
+        printf("History entry: ");
+        while (bp > 0) {
+            fsg_hist_entry_t *hist_entry = fsg_history_entry_get(h, bp);
+	    fsg_link_t *fl = fsg_hist_entry_fsglink(hist_entry);
+    	    char const *baseword;
+    	    int32 wid;
+    	    bp = fsg_hist_entry_pred(hist_entry);
+    	    wid = fsg_link_wid(fl);
+
+    	    if (fl == NULL)
+        	    continue;
+
+    	    baseword = fsg_model_word_str(h->fsg, wid);
+
+    	    printf("%s(%d->%d:%d) ", baseword, 
+    				     fsg_link_from_state(hist_entry->fsglink), 
+    				     fsg_link_to_state(hist_entry->fsglink), 
+    				     hist_entry->frame);
+	}
+	printf("\n");
+    }
+}
