@@ -54,9 +54,10 @@
 #include <assert.h>
 
 /* SphinxBase headers. */
-#include <err.h>
-#include <ckd_alloc.h>
-#include <cmd_ln.h>
+#include <sphinxbase/err.h>
+#include <sphinxbase/ckd_alloc.h>
+#include <sphinxbase/strfuncs.h>
+#include <sphinxbase/cmd_ln.h>
 
 /* Local headers. */
 #include "pocketsphinx_internal.h"
@@ -111,23 +112,28 @@ fsg_search_init(cmd_ln_t *config,
     fsgs->frame = -1;
 
     /* Initialize FSG table. */
-    fsgs->fsgs = hash_table_new(5, FALSE);
+    fsgs->fsgs = hash_table_new(5, HASH_CASE_YES);
 
     /* Get search pruning parameters */
     fsgs->beam_factor = 1.0f;
     fsgs->beam = fsgs->beam_orig
-        = (int32) logmath_log(acmod->lmath, cmd_ln_float64_r(config, "-beam"));
+        = (int32) logmath_log(acmod->lmath, cmd_ln_float64_r(config, "-beam"))
+        >> SENSCR_SHIFT;
     fsgs->pbeam = fsgs->pbeam_orig
-        = (int32) logmath_log(acmod->lmath, cmd_ln_float64_r(config, "-pbeam"));
+        = (int32) logmath_log(acmod->lmath, cmd_ln_float64_r(config, "-pbeam"))
+        >> SENSCR_SHIFT;
     fsgs->wbeam = fsgs->wbeam_orig
-        = (int32) logmath_log(acmod->lmath, cmd_ln_float64_r(config, "-wbeam"));
+        = (int32) logmath_log(acmod->lmath, cmd_ln_float64_r(config, "-wbeam"))
+        >> SENSCR_SHIFT;
 
     /* LM related weights/penalties */
     fsgs->lw = cmd_ln_float32_r(config, "-lw");
     fsgs->pip = (int32) (logmath_log(acmod->lmath, cmd_ln_float32_r(config, "-pip"))
-                           * fsgs->lw);
+                           * fsgs->lw)
+        >> SENSCR_SHIFT;
     fsgs->wip = (int32) (logmath_log(acmod->lmath, cmd_ln_float32_r(config, "-wip"))
-                           * fsgs->lw);
+                           * fsgs->lw)
+        >> SENSCR_SHIFT;
 
     /* Best path search (and confidence annotation)? */
     if (cmd_ln_boolean_r(config, "-bestpath"))
@@ -170,11 +176,7 @@ fsg_search_init(cmd_ln_t *config,
         /* Take the -toprule if specified. */
         if ((toprule = cmd_ln_str_r(config, "-toprule"))) {
             char *anglerule;
-
-            anglerule = ckd_calloc(1, strlen(toprule) + 3);
-            anglerule[0] = '<';
-            strcat(anglerule, toprule);
-            strcat(anglerule, ">");
+            anglerule = string_join("<", toprule, ">", NULL);
             rule = jsgf_get_rule(fsgs->jsgf, anglerule);
             ckd_free(anglerule);
             if (rule == NULL) {
@@ -189,8 +191,10 @@ fsg_search_init(cmd_ln_t *config,
             for (itor = jsgf_rule_iter(fsgs->jsgf); itor;
                  itor = jsgf_rule_iter_next(itor)) {
                 rule = jsgf_rule_iter_rule(itor);
-                if (jsgf_rule_public(rule))
+                if (jsgf_rule_public(rule)) {
+            	    jsgf_rule_iter_free(itor);
                     break;
+                }
             }
             if (rule == NULL) {
                 E_ERROR("No public rules found in %s\n", path);
@@ -227,15 +231,19 @@ fsg_search_free(ps_search_t *search)
     if (fsgs->jsgf)
         jsgf_grammar_free(fsgs->jsgf);
     fsg_lextree_free(fsgs->lextree);
-    fsg_history_reset(fsgs->history);
-    fsg_history_set_fsg(fsgs->history, NULL, NULL);
-    for (itor = hash_table_iter(fsgs->fsgs);
-         itor; itor = hash_table_iter_next(itor)) {
-        fsg_model_t *fsg = (fsg_model_t *) hash_entry_val(itor->ent);
-        fsg_model_free(fsg);
+    if (fsgs->history) {
+        fsg_history_reset(fsgs->history);
+        fsg_history_set_fsg(fsgs->history, NULL, NULL);
+        fsg_history_free(fsgs->history);
+    }
+    if (fsgs->fsgs) {
+        for (itor = hash_table_iter(fsgs->fsgs);
+             itor; itor = hash_table_iter_next(itor)) {
+            fsg_model_t *fsg = (fsg_model_t *) hash_entry_val(itor->ent);
+            fsg_model_free(fsg);
+        }
+        hash_table_free(fsgs->fsgs);
     }
-    hash_table_free(fsgs->fsgs);
-    fsg_history_free(fsgs->history);
     hmm_context_free(fsgs->hmmctx);
     ckd_free(fsgs);
 }
@@ -251,6 +259,10 @@ fsg_search_reinit(ps_search_t *search, dict_t *dict, dict2pid_t *d2p)
 
     /* Free old dict2pid, dict */
     ps_search_base_reinit(search, dict, d2p);
+    
+    /* Nothing to update */
+    if (fsgs->fsg == NULL)
+	return 0;
 
     /* Update the number of words (not used by this module though). */
     search->n_words = dict_size(dict);
@@ -346,12 +358,12 @@ fsg_search_add_altpron(fsg_search_t *fsgs, fsg_model_t *fsg)
         wid = dict_wordid(dict, word);
         if (wid != BAD_S3WID) {
             while ((wid = dict_nextalt(dict, wid)) != BAD_S3WID) {
-	        fsg_model_add_alt(fsg, word, dict_wordstr(dict, wid));
-    	        ++n_alt;
+	        n_alt += fsg_model_add_alt(fsg, word, dict_wordstr(dict, wid));
     	    }
     	}
     }
 
+    E_INFO("Added %d alternate word transitions\n", n_alt);
     return n_alt;
 }
 
@@ -729,7 +741,7 @@ fsg_search_null_prop(fsg_search_t *fsgs)
     int32 bpidx, n_entries, thresh, newscore;
     fsg_hist_entry_t *hist_entry;
     fsg_link_t *l;
-    int32 s, d;
+    int32 s;
     fsg_model_t *fsg;
 
     fsg = fsgs->fsg;
@@ -738,6 +750,7 @@ fsg_search_null_prop(fsg_search_t *fsgs)
     n_entries = fsg_history_n_entries(fsgs->history);
 
     for (bpidx = fsgs->bpidx_start; bpidx < n_entries; bpidx++) {
+        fsg_arciter_t *itor;
         hist_entry = fsg_history_entry_get(fsgs->history, bpidx);
 
         l = fsg_hist_entry_fsglink(hist_entry);
@@ -750,22 +763,25 @@ fsg_search_null_prop(fsg_search_t *fsgs)
          * propagate one step, since FSG contains transitive closure of null
          * transitions.)
          */
-        for (d = 0; d < fsg_model_n_state(fsg); d++) {
-            l = fsg_model_null_trans(fsg, s, d);
+        /* Add all links from from_state to dst */
+        for (itor = fsg_model_arcs(fsg, s); itor;
+             itor = fsg_arciter_next(itor)) {
+            fsg_link_t *l = fsg_arciter_get(itor);
 
-            if (l) {            /* Propagate history entry through this null transition */
-                newscore =
-                    fsg_hist_entry_score(hist_entry) +
-                    fsg_link_logs2prob(l);
+            /* FIXME: Need to deal with tag transitions somehow. */
+            if (fsg_link_wid(l) != -1)
+                continue;
+            newscore =
+                fsg_hist_entry_score(hist_entry) +
+                (fsg_link_logs2prob(l) >> SENSCR_SHIFT);
 
-                if (newscore >= thresh) {
-                    fsg_history_entry_add(fsgs->history, l,
-                                          fsg_hist_entry_frame(hist_entry),
-                                          newscore,
-                                          bpidx,
-                                          fsg_hist_entry_lc(hist_entry),
-                                          fsg_hist_entry_rc(hist_entry));
-                }
+            if (newscore >= thresh) {
+                fsg_history_entry_add(fsgs->history, l,
+                                      fsg_hist_entry_frame(hist_entry),
+                                      newscore,
+                                      bpidx,
+                                      fsg_hist_entry_lc(hist_entry),
+                                      fsg_hist_entry_rc(hist_entry));
             }
         }
     }
@@ -1021,21 +1037,11 @@ fsg_search_finish(ps_search_t *search)
          (fsgs->frame > 0) ? fsgs->n_sen_eval / fsgs->frame : 0,
          n_hist, (fsgs->frame > 0) ? n_hist / fsgs->frame : 0);
 
-    /* Sanity check */
-    if (fsgs->n_hmm_eval >
-        fsg_lextree_n_pnode(fsgs->lextree) * fsgs->frame) {
-        E_ERROR
-            ("SANITY CHECK #HMMEval(%d) > %d (#HMMs(%d)*#frames(%d)) FAILED\n",
-             fsgs->n_hmm_eval,
-             fsg_lextree_n_pnode(fsgs->lextree) * fsgs->frame,
-             fsg_lextree_n_pnode(fsgs->lextree), fsgs->frame);
-    }
-
     return 0;
 }
 
 static int
-fsg_search_find_exit(fsg_search_t *fsgs, int frame_idx, int final, int32 *out_score)
+fsg_search_find_exit(fsg_search_t *fsgs, int frame_idx, int final, int32 *out_score, int32* out_is_final)
 {
     fsg_hist_entry_t *hist_entry;
     fsg_model_t *fsg;
@@ -1070,8 +1076,14 @@ fsg_search_find_exit(fsg_search_t *fsgs, int frame_idx, int final, int32 *out_sc
 
         fl = fsg_hist_entry_fsglink(hist_entry);
         score = fsg_hist_entry_score(hist_entry);
+        
+        if (fl == NULL)
+	    break;
 
-        if (score BETTER_THAN bestscore) {
+	/* Prefer final hypothesis */
+	if (score == bestscore && fsg_link_to_state(fl) == fsg_model_final_state(fsg)) {
+    	    besthist = bpidx;
+	} else if (score BETTER_THAN bestscore) {
             /* Only enforce the final state constraint if this is a final hypothesis. */
             if ((!final)
                 || fsg_link_to_state(fl) == fsg_model_final_state(fsg)) {
@@ -1079,7 +1091,7 @@ fsg_search_find_exit(fsg_search_t *fsgs, int frame_idx, int final, int32 *out_sc
                 besthist = bpidx;
             }
         }
-
+        
         --bpidx;
         if (bpidx < 0)
             break;
@@ -1089,13 +1101,19 @@ fsg_search_find_exit(fsg_search_t *fsgs, int frame_idx, int final, int32 *out_sc
 
     /* Final state not reached. */
     if (besthist == -1) {
-        E_ERROR("Final state not reached in frame %d\n", frame_idx);
+        E_ERROR("Final result does not match the grammar in frame %d\n", frame_idx);
         return -1;
     }
 
     /* This here's the one we want. */
     if (out_score)
         *out_score = bestscore;
+    if (out_is_final) {
+	fsg_link_t *fl;
+	hist_entry = fsg_history_entry_get(fsgs->history, besthist);
+	fl = fsg_hist_entry_fsglink(hist_entry);
+	*out_is_final = (fsg_link_to_state(fl) == fsg_model_final_state(fsg));
+    }
     return besthist;
 }
 
@@ -1121,7 +1139,7 @@ fsg_search_bestpath(ps_search_t *search, int32 *out_score, int backward)
 }
 
 char const *
-fsg_search_hyp(ps_search_t *search, int32 *out_score)
+fsg_search_hyp(ps_search_t *search, int32 *out_score, int32 *out_is_final)
 {
     fsg_search_t *fsgs = (fsg_search_t *)search;
     dict_t *dict = ps_search_dict(search);
@@ -1130,7 +1148,7 @@ fsg_search_hyp(ps_search_t *search, int32 *out_score)
     int bp, bpidx;
 
     /* Get last backpointer table index. */
-    bpidx = fsg_search_find_exit(fsgs, fsgs->frame, fsgs->final, out_score);
+    bpidx = fsg_search_find_exit(fsgs, fsgs->frame, fsgs->final, out_score, out_is_final);
     /* No hypothesis (yet). */
     if (bpidx <= 0)
         return NULL;
@@ -1140,10 +1158,14 @@ fsg_search_hyp(ps_search_t *search, int32 *out_score)
         ps_lattice_t *dag;
         ps_latlink_t *link;
 
-        if ((dag = fsg_search_lattice(search)) == NULL)
+        if ((dag = fsg_search_lattice(search)) == NULL) {
+    	    E_WARN("Failed to obtain the lattice while bestpath enabled\n");
             return NULL;
-        if ((link = fsg_search_bestpath(search, out_score, FALSE)) == NULL)
+        }
+        if ((link = fsg_search_bestpath(search, out_score, FALSE)) == NULL) {
+    	    E_WARN("Failed to find the bestpath in a lattice\n");
             return NULL;
+        }
         return ps_lattice_hyp(dag, link);
     }
 
@@ -1260,7 +1282,7 @@ fsg_search_seg_iter(ps_search_t *search, int32 *out_score)
     fsg_seg_t *itor;
     int bp, bpidx, cur;
 
-    bpidx = fsg_search_find_exit(fsgs, fsgs->frame, fsgs->final, out_score);
+    bpidx = fsg_search_find_exit(fsgs, fsgs->frame, fsgs->final, out_score, NULL);
     /* No hypothesis (yet). */
     if (bpidx <= 0)
         return NULL;
@@ -1335,13 +1357,22 @@ fsg_search_prob(ps_search_t *search)
 }
 
 static ps_latnode_t *
-new_node(ps_lattice_t *dag, fsg_model_t *fsg, int sf, int ef, int32 wid, int32 ascr)
+find_node(ps_lattice_t *dag, fsg_model_t *fsg, int sf, int32 wid, int32 node_id)
 {
     ps_latnode_t *node;
 
     for (node = dag->nodes; node; node = node->next)
-        if (node->sf == sf && node->wid == wid)
+        if ((node->sf == sf) && (node->wid == wid) && (node->node_id == node_id))
             break;
+    return node;
+}
+
+static ps_latnode_t *
+new_node(ps_lattice_t *dag, fsg_model_t *fsg, int sf, int ef, int32 wid, int32 node_id, int32 ascr)
+{
+    ps_latnode_t *node;
+
+    node = find_node(dag, fsg, sf, wid, node_id);
 
     if (node) {
         /* Update end frames. */
@@ -1363,25 +1394,16 @@ new_node(ps_lattice_t *dag, fsg_model_t *fsg, int sf, int ef, int32 wid, int32 a
         node->entries = NULL;
         node->exits = NULL;
         node->info.best_exit = ascr;
+        node->node_id = node_id;
 
         node->next = dag->nodes;
         dag->nodes = node;
+        ++dag->n_nodes;
     }
 
     return node;
 }
 
-static ps_latnode_t *
-find_node(ps_lattice_t *dag, fsg_model_t *fsg, int sf, int32 wid)
-{
-    ps_latnode_t *node;
-
-    for (node = dag->nodes; node; node = node->next)
-        if (node->sf == sf && node->wid == wid)
-            break;
-    return node;
-}
-
 static ps_latnode_t *
 find_start_node(fsg_search_t *fsgs, ps_lattice_t *dag)
 {
@@ -1413,7 +1435,7 @@ find_start_node(fsg_search_t *fsgs, ps_lattice_t *dag)
         wid = fsg_model_word_add(fsgs->fsg, "<s>");
         if (fsgs->fsg->silwords)
             bitvec_set(fsgs->fsg->silwords, wid);
-        node = new_node(dag, fsgs->fsg, 0, 0, wid, 0);
+        node = new_node(dag, fsgs->fsg, 0, 0, wid, -1, 0);
         for (st = start; st; st = gnode_next(st))
             ps_lattice_link(dag, node, gnode_ptr(st), 0, 0);
     }
@@ -1466,11 +1488,10 @@ find_end_node(fsg_search_t *fsgs, ps_lattice_t *dag)
          * out of all of them. */
         gnode_t *st;
         int wid;
-
         wid = fsg_model_word_add(fsgs->fsg, "</s>");
         if (fsgs->fsg->silwords)
             bitvec_set(fsgs->fsg->silwords, wid);
-        node = new_node(dag, fsgs->fsg, fsgs->frame, fsgs->frame, wid, 0);
+        node = new_node(dag, fsgs->fsg, fsgs->frame, fsgs->frame, wid, -1, 0);
         /* Use the "best" (in reality it will be the only) exit link
          * score from this final node as the link score. */
         for (st = end; st; st = gnode_next(st)) {
@@ -1546,7 +1567,6 @@ fsg_search_lattice(ps_search_t *search)
     n = fsg_history_n_entries(fsgs->history);
     for (i = 0; i < n; ++i) {
         fsg_hist_entry_t *fh = fsg_history_entry_get(fsgs->history, i);
-        ps_latnode_t *node;
         int32 ascr;
         int sf;
 
@@ -1578,7 +1598,7 @@ fsg_search_lattice(ps_search_t *search)
          * destination node, and thus we need to preserve its score in
          * case it turns out to be utterance-final.
          */
-        node = new_node(dag, fsg, sf, fh->frame, fh->fsglink->wid, ascr);
+        new_node(dag, fsg, sf, fh->frame, fh->fsglink->wid, fsg_link_to_state(fh->fsglink), ascr);
     }
 
     /*
@@ -1587,10 +1607,10 @@ fsg_search_lattice(ps_search_t *search)
     n = fsg_history_n_entries(fsgs->history);
     for (i = 0; i < n; ++i) {
         fsg_hist_entry_t *fh = fsg_history_entry_get(fsgs->history, i);
+        fsg_arciter_t *itor;
         ps_latnode_t *src, *dest;
         int32 ascr;
         int sf;
-        int j;
 
         /* Skip null transitions. */
         if (fh->fsglink == NULL || fh->fsglink->wid == -1)
@@ -1606,47 +1626,57 @@ fsg_search_lattice(ps_search_t *search)
             ascr = fh->score;
             sf = 0;
         }
-        src = find_node(dag, fsg, sf, fh->fsglink->wid);
-    
-        /*
-         * For each non-epsilon link following this one, look for a
-         * matching node in the lattice and link to it.
-         */
+        src = find_node(dag, fsg, sf, fh->fsglink->wid, fsg_link_to_state(fh->fsglink));
         sf = fh->frame + 1;
-        for (j = 0; j < fsg->n_state; ++j) {
-            gnode_t *gn;
 
-            for (gn = fsg_model_trans(fsg, fh->fsglink->to_state, j); gn; gn = gnode_next(gn)) {
-                fsg_link_t *link = gnode_ptr(gn);
-                if ((dest = find_node(dag, fsg, sf, link->wid)) != NULL)
-                    ps_lattice_link(dag, src, dest, ascr, fh->frame);
+        for (itor = fsg_model_arcs(fsg, fsg_link_to_state(fh->fsglink));
+             itor; itor = fsg_arciter_next(itor)) {
+            fsg_link_t *link = fsg_arciter_get(itor);
+            
+            /* FIXME: Need to figure out what to do about tag transitions. */
+            if (link->wid >= 0) {
+                /*
+                 * For each non-epsilon link following this one, look for a
+                 * matching node in the lattice and link to it.
+                 */
+                if ((dest = find_node(dag, fsg, sf, link->wid, fsg_link_to_state(link))) != NULL)
+            	    ps_lattice_link(dag, src, dest, ascr, fh->frame);
             }
-
-            /*
-             * Transitive closure on nulls has already been done, so we
-             * just need to look one link forward from them.
-             */
-            if (fsg_model_null_trans(fsg, fh->fsglink->to_state,j)) {
-                gnode_t *gn2;
-                int k;
-
+            else {
+                /*
+                 * Transitive closure on nulls has already been done, so we
+                 * just need to look one link forward from them.
+                 */
+                fsg_arciter_t *itor2;
+                
                 /* Add all non-null links out of j. */
-                for (k = 0; k < fsg->n_state; ++k) {
-                    for (gn2 = fsg_model_trans(fsg, j, k); gn2; gn2 = gnode_next(gn2)) {
-                        fsg_link_t *link = gnode_ptr(gn2);
-                        if ((dest = find_node(dag, fsg, sf, link->wid)) != NULL)
-                            ps_lattice_link(dag, src, dest, ascr, fh->frame);
+                for (itor2 = fsg_model_arcs(fsg, fsg_link_to_state(link));
+                     itor2; itor2 = fsg_arciter_next(itor2)) {
+                    fsg_link_t *link = fsg_arciter_get(itor2);
+
+                    if (link->wid == -1)
+                        continue;
+                    
+                    if ((dest = find_node(dag, fsg, sf, link->wid, fsg_link_to_state(link))) != NULL) {
+                        ps_lattice_link(dag, src, dest, ascr, fh->frame);
                     }
                 }
             }
         }
     }
 
+
     /* Figure out which nodes are the start and end nodes. */
-    if ((dag->start = find_start_node(fsgs, dag)) == NULL)
+    if ((dag->start = find_start_node(fsgs, dag)) == NULL) {
+	E_WARN("Failed to find the start node\n");
         goto error_out;
-    if ((dag->end = find_end_node(fsgs, dag)) == NULL)
+    }
+    if ((dag->end = find_end_node(fsgs, dag)) == NULL) {
+	E_WARN("Failed to find the end node\n");
         goto error_out;
+    }
+
+
     E_INFO("lattice start node %s.%d end node %s.%d\n",
            fsg_model_word_str(fsg, dag->start->wid), dag->start->sf,
            fsg_model_word_str(fsg, dag->end->wid), dag->end->sf);
@@ -1675,15 +1705,19 @@ fsg_search_lattice(ps_search_t *search)
 
         silpen = (int32)(logmath_log(fsg->lmath,
                                      cmd_ln_float32_r(ps_search_config(fsgs), "-silprob"))
-                         * fsg->lw);
+                         * fsg->lw)
+            >> SENSCR_SHIFT;
         fillpen = (int32)(logmath_log(fsg->lmath,
                                       cmd_ln_float32_r(ps_search_config(fsgs), "-fillprob"))
-                          * fsg->lw);
+                          * fsg->lw)
+            >> SENSCR_SHIFT;
         ps_lattice_bypass_fillers(dag, silpen, fillpen);
     }
     search->dag = dag;
+
     return dag;
 
+
 error_out:
     ps_lattice_free(dag);
     return NULL;
