@@ -67,7 +67,7 @@ main(int argc, char *argv[])
 		}
 		ngram_fwdtree_finish(ngs);
 		printf("FWDTREE: %s\n",
-		       ngram_search_bp_hyp(ngs, ngram_search_find_exit(ngs, -1, NULL)));
+		       ngram_search_bp_hyp(ngs, ngram_search_find_exit(ngs, -1, NULL, NULL)));
 
 		TEST_ASSERT(acmod_end_utt(acmod) >= 0);
 		fclose(rawfh);
@@ -96,11 +96,12 @@ main(int argc, char *argv[])
 		ckd_free(besthyp);
 	}
 	TEST_EQUAL(0, strcmp("go forward ten years",
-			     ngram_search_bp_hyp(ngs, ngram_search_find_exit(ngs, -1, NULL))));
+			     ngram_search_bp_hyp(ngs, ngram_search_find_exit(ngs, -1, NULL, NULL))));
 	c = clock() - c;
 	printf("5 * fwdtree + bestpath + N-best search in %.2f sec\n",
 	       (double)c / CLOCKS_PER_SEC);
 	ps_free(ps);
+	cmd_ln_free_r(config);
 
 	return 0;
 }
